﻿using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace TestProject;

/// <summary>
/// 등고선 차트 컨트롤
/// </summary>
public partial class ContourChartControl : UserControl
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 레벨 배열
    /// </summary>
    private float[] levelArray;

    /// <summary>
    /// 색상 배열
    /// </summary>
    private Color[] colorArray;

    /// <summary>
    /// 데이터 배열
    /// </summary>
    private float[,] valueArray;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - ContourChartControl()

    /// <summary>
    /// 생성자
    /// </summary>
    public ContourChartControl()
    {
        InitializeComponent();
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 데이터 설정하기 - SetData(levelArray, colorArray, valueArray)

    /// <summary>
    /// 데이터 설정하기
    /// </summary>
    /// <param name="levelArray">레벨 배열</param>
    /// <param name="colorArray">색상 배열</param>
    /// <param name="valueArray">값 배열</param>
    public void SetData(float[] levelArray, Color[] colorArray, float[,] valueArray)
    {
        this.levelArray = levelArray;
        this.colorArray = colorArray;
        this.valueArray = valueArray;

        Invalidate();
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Protected

    #region 페인트 처리하기 - OnPaint(e)

    /// <summary>
    /// 페인트 처리하기
    /// </summary>
    /// <param name="e">이벤트 인자</param>
    protected override void OnPaint(PaintEventArgs e)
    {
        base.OnPaint(e);

        if(this.valueArray == null || this.levelArray == null || this.colorArray == null)
        {
            return;
        }

        DrawContour(e.Graphics);
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region 포인트 스케일 설정하기 - ScalePoint(sourcePoint)

    /// <summary>
    /// 포인트 스케일 설정하기
    /// </summary>
    /// <param name="sourcePoint">소스 포인트</param>
    /// <returns>스케일 포인트</returns>
    private PointF ScalePoint(PointF sourcePoint)
    {
        return new PointF(sourcePoint.X * Width / (valueArray.GetLength(0) - 1), Height - sourcePoint.Y * Height / (valueArray.GetLength(1) - 1));
    }

    #endregion
    #region 등고선 셀 그리기 - DrawContourCell(graphics, x, y, valueArray, lowLevel, highLevel, color)

    /// <summary>
    /// 등고선 셀 그리기
    /// </summary>
    /// <param name="graphics">그래픽스</param>
    /// <param name="x">X</param>
    /// <param name="y">Y</param>
    /// <param name="valueArray">값 배열</param>
    /// <param name="lowLevel">하한 레벨</param>
    /// <param name="highLevel">상한 레벨</param>
    /// <param name="color">색상</param>
    private void DrawContourCell(Graphics graphics, int x, int y, float[] valueArray, float lowLevel, float highLevel, Color color)
    {
        List<PointF> pointList = new List<PointF>();

        for(int i = 0; i < 4; i++)
        {
            int j = (i + 1) % 4;

            if((valueArray[i] < lowLevel && valueArray[j] >= lowLevel) || (valueArray[i] >= lowLevel && valueArray[j] < lowLevel))
            {
                float t = (lowLevel - valueArray[i]) / (valueArray[j] - valueArray[i]);

                pointList.Add
                (
                    new PointF
                    (
                        x + (i % 2) + t * ((j % 2) - (i % 2)),
                        y + (i / 2) + t * ((j / 2) - (i / 2))
                    )
                );
            }
        }

        if(pointList.Count == 2)
        {
            using(Pen pen = new Pen(color, 1))
            {
                graphics.DrawLine(pen, ScalePoint(pointList[0]), ScalePoint(pointList[1]));
            }
        }
    }

    #endregion
    #region 등고선 그리기 - DrawContour(graphics)

    /// <summary>
    /// 등고선 그리기
    /// </summary>
    /// <param name="graphics">그래픽스</param>
    private void DrawContour(Graphics graphics)
    {
        int width  = this.valueArray.GetLength(0);
        int height = this.valueArray.GetLength(1);

        for(int x = 0; x < width - 1; x++)
        {
            for(int y = 0; y < height - 1; y++)
            {
                float[] cellValueArray = new float[]
                {
                    valueArray[x    , y    ],
                    valueArray[x + 1, y    ],
                    valueArray[x + 1, y + 1],
                    valueArray[x    , y + 1]
                };

                for(int i = 0; i < this.levelArray.Length - 1; i++)
                {
                    DrawContourCell(graphics, x, y, cellValueArray, this.levelArray[i], this.levelArray[i + 1], this.colorArray[i]);
                }
            }
        }
    }

    #endregion
}